#!/usr/bin/sh

# 
# File: audio.sh
# Version: 0.5
# Authors: Stéphane Cousot <http://www.ubaa.net/>
# License: audio.sh is released under GNU General Public License, version 3 or later
#
#	- value to dB   = ( value-(Math.abs(min)/step) )*step
# 	- dB to value   = ( dB+Math.abs(min) ) / step
#	- db to Percent = ((dB*100)-min)/(max-main) * 100
#
# --------------------------------------------------------------------------------------
#
# This file is part of the Locus Sonus Streambox Player project.
# http://locusonus.org/streambox/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#



if [ `whoami` != 'root' ]; then # && [ `whoami` != 'http' ]; then
	echo "Permission denied"
	echo "you must be root to run this script."
	exit 1
fi



# load (copy first) configuration file
if [ ! -f /boot/settings.txt ]; then
	echo "Copy default configuration"
	cp /home/locusonus/www/conf/default /boot/settings.txt
fi
source /boot/settings.txt




docheck()
{
	case $@ in
		onboard )
			BIN="aplay"
			CARD="ALSA"
			NAME="bcm2835 ALSA"
			;;
		respeaker )
			BIN="aplay"
			CARD="seeed2micvoicec"
			NAME="ReSpeaker 2-Mics"
			;;
		icicle )
			BIN="arecord"
			CARD="Icicle"
			NAME="Blue Icicle"
			;;
		cirrus )
			BIN="arecord"
			CARD="RPiCirrus"
			NAME="Cirrus Logic/Wolfson"
			;;
		* )
			usage
			exit 1
			;;
	esac


	CHECK=`$BIN -l 2>/dev/null | grep -E "$CARD"`
	if [ "$CHECK" ]; then 
		echo "the $NAME audio card is connected"
	fi
}

dotest()
{
	#stop running test
	PROC=`ps x -o  "%r %c" | grep "speaker-test" | head -n 1`
	if [ "$PROC" ]; then 
		PGID=`echo -e "${PROC}" | sed -e 's/^[[:space:]]*//' | cut -d' ' -f1`
		if [ "$PGID" != "$$" ]; then kill -TERM -$PGID; fi
	fi

	if [ "$1" == "start" ]; then
		if [[ -z $2 ]]; then usage;
		else speaker-test -t $2 -l 0 -c 2; fi
	fi
}

domonitoring()
{
	if [ "$1" == "stop" ]; then 
		PROC=`ps x -o  "%r %c" | grep "audio.sh" | head -n 1`
		if [ "$PROC" ]; then 
			PGID=`echo -e "${PROC}" | sed -e 's/^[[:space:]]*//' | cut -d' ' -f1`
			if [ "$PGID" != "$$" ]; then kill -TERM -$PGID; fi
		fi
		exit 0
	fi

	NDEV=`arecord -l | grep "card" | wc -l`
	if [ "$SOUNDCARD" -gt "$NDEV" ]; then SOUNDCARD=`arecord -l | pcregrep -o1 -e "card (\d+)" | head -n 1`; fi
	if [[ "$VIRTUAL_STEREO" == "yes" && "$NDEV" != "1" ]]; then IN="virtual";
	else IN="hw:${SOUNDCARD:-1},0"; fi

	OUT=`aplay -l | pcregrep -o1 -e "card (\d+)" | tail -1`
	FORMAT=`arecord -D $IN --dump-hw-params 2>&1 | tail -1 | cut -d ' ' -f2`
	CHANNELS="$AUDIO_CHANNEL"

	# force DAC stereo only
	OUT_CHANNELS=`aplay -D hw:$OUT,0 --dump-hw-params -d1 /dev/zero 2>&1 | grep "CHANNELS"`
	if [[ -z `echo $OUT_CHANNELS | grep "$CHANNELS"` ]]; then CHANNELS=`echo $OUT_CHANNELS | pcregrep -o1 -e "CHANNELS.*(\d+)"`; fi

	arecord -D $IN -r $AUDIO_SAMPLERATE -c $CHANNELS -f $FORMAT | \
	aplay -D hw:$OUT,0 -r $AUDIO_SAMPLERATE -c $CHANNELS -f $FORMAT
}


# args db min [step]
dbToValue()
{ 
	DB="$1"
	MIN=`awk "BEGIN{x=$2; print x<0?-x:x}"`
	STEP="$3"

	if [[ -z $STEP ]]; then STEP="1"; fi
	awk "BEGIN {print ($DB+$MIN)/$STEP}"
}

valueToDB()
{

	VAL="$1"
	MIN=`awk "BEGIN{x=$2; print x<0?-x:x}"`
	STEP="$3"

	if [[ -z $STEP ]]; then STEP="1"; fi
	awk "BEGIN {print ($VAL-($MIN/$STEP))*$STEP}"
}

# FIXEME
dbToPercent()
{
	DB="$1"
	MIN="$2"
	MAX="$3"

	awk "BEGIN {printf(\"%.2f\", ((($DB*100)-($MIN))/(($MAX)-($MIN)))*100) }"
}

printDB()
{
	printf "%+.2f dB\n" $1 | sed 's/[+-]/\0 /'
}

join()
{
	perl -e '$s = shift @ARGV; print join($s, @ARGV);' "$@"; 
}




# ----------------------------------------------------------------------------------------------------------------------
#	ALSA / AUDIO SETUP
# ----------------------------------------------------------------------------------------------------------------------

configure()
{

	if [ "$1" == "quiet" ]; then OPT="-q"; fi

	setmaster $OPT $DEFAULT_AUDIO_OUTPUT_VOLUME
	asound_conf $OPT $SOUNDCARD


	HAVE_ALSA=`docheck onboard`
	if [ "$HAVE_ALSA" ]; then
	 	onboard_config $OPT
	fi

	HAVE_RESPEAKER=`docheck respeaker`
	if [ "$HAVE_RESPEAKER" ]; then
		respeaker_config $OPT
	fi

	HAVE_ICICLE=`docheck icicle`
	if [ "$HAVE_ICICLE" ]; then
		icicle_config $OPT
	fi

	HAVE_CIRRUS=`docheck cirrus`
	if [ "$HAVE_CIRRUS" ]; then
		cirrus_config $OPT
	fi
}

setmaster()
{

	VALUE=`echo "${@: -1}"`
	CHECK=`aplay -l 2>/dev/null`

	if [ "$1" != "-q" ]; then
		echo "-- set ALSA master to $VALUE"
	fi

	if [[ -z "$CHECK" && "$1" != "-q" ]]; then 
		echo "no soundcards found"
		return
	fi

	CHECK=`amixer scontrols | grep "'Master'"`
	if [ "$CHECK" ]; then amixer -q sset Master $VALUE; fi

	CHECK=`amixer scontrols | grep "'Speaker'"`
	if [ "$CHECK" ]; then amixer -q sset Speaker $VALUE; fi

	CHECK=`amixer scontrols | grep "'Headphone'"`
	if [ "$CHECK" ]; then amixer -q sset Headphone $VALUE; fi
}

asound_conf()
{
	SOUNDCARD=`echo "${@: -1}"`
	PCM_ROUTING=('program')

	pcm_config

	if [ "$1" != "-q" ]; then 
		echo "-- set ALSA audio card to $SOUNDCARD"
		echo "-- route audio signal: `join ' > ' ${PCM_ROUTING[@]}`"
	fi

	cat << EOF > /etc/asound.conf 
pcm.!default {
    type plug
    slave.pcm $PCM_SLAVE
}

ctl.!default {
    type hw
    card $SOUNDCARD
}

pcm.soundcard {
   type hw
   card $SOUNDCARD
}
EOF

	# add required plugin
	if [ "$1" == "-q" ]; then plugins_config "-q";
	else plugins_config; fi

}

pcm_config()
{

	# check soundcard
	CHECK=`aplay -l 2>/dev/null | grep "card $SOUNDCARD"`
	if [ -z "$CHECK" ]; then SOUNDCARD=0; fi


	if [ "$AUDIO_COMPRESSOR" == "yes" ]; then
		PCM_SLAVE="compressor"
		PCM_ROUTING+=("compressor")
		PCM_COMPRESSOR_PLUGIN="plughw:$SOUNDCARD,0"
	fi

	if [ "$AUDIO_LIMITER" == "yes" ]; then
		
		if [ -z "$PCM_SLAVE" ]; then PCM_SLAVE="limiter"; fi

		PCM_ROUTING+=("limiter")
		PCM_COMPRESSOR_PLUGIN="plugin_limiter"
		PCM_LIMITER_PLUGIN="plughw:$SOUNDCARD,0"
	fi
	
	if [ "$AUDIO_EQUALIZER" == "yes" ]; then

		if [ -z "$PCM_SLAVE" ]; then PCM_SLAVE="equalizer"; fi
		if [ "$AUDIO_LIMITER" == "no" ]; then PCM_COMPRESSOR_PLUGIN="plugin_equalizer_f"; fi

		PCM_ROUTING+=("equalizer")
		PCM_LIMITER_PLUGIN="plugin_equalizer_f"
		PCM_EQUALIZER_PLUGIN="plughw:$SOUNDCARD,0"
	fi


	# default
	if [ -z "$PCM_SLAVE" ]; then PCM_SLAVE="soundcard"; fi
	PCM_ROUTING+=("soundcard")
}




# ----------------------------------------------------------------------------------------------------------------------
#	AUDIO PLUGIN
#
#   LADSPA plugin library : http://www.ladspa.org/
#   LADSPA documentation  : http://plugin.org.uk/ladspa-swh/docs/ladspa-swh.html
#   LADSPA source         : https://github.com/swh/ladspa/
# ----------------------------------------------------------------------------------------------------------------------

doplugin()
{
	do=$1
	shift

	case $do in
		setup )
			setcard $SOUNDCARD
			;;
		reset )
			plugin_reset_eq
			;;
		equalizer|eq )
			plugin_conf_eq $@
			;;
		compressor )
			echo "LIVE UPDATE NOT IMPLEMENTED !"
			#plugin_set_compressor $@
			;;
		limiter )
			echo "LIVE UPDATE NOT IMPLEMENTED !"
			#plugin_set_limiter $@
			;;
		* )
			usage
			;;
	esac
}

plugins_config()
{
	if [ "$AUDIO_COMPRESSOR" == "yes" ]; then
		plugin_add_compressor $1 $AUDIO_COMPRESSOR_PEAK_LIMIT $AUDIO_COMPRESSOR_RELEASE_TIME $AUDIO_COMPRESSOR_FAST_RATIO $AUDIO_COMPRESSOR_RATIO
	fi
	if [ "$AUDIO_LIMITER" == "yes" ]; then
		plugin_add_limiter $1 $AUDIO_LIMITER_IN_GAIN $AUDIO_LIMITER_LIMIT $AUDIO_LIMITER_RELEASE_TIME
	fi
	if [ "$AUDIO_EQUALIZER" == "yes" ]; then
		plugin_add_eq $AUDIO_EQ_BAND_50 $AUDIO_EQ_BAND_100 $AUDIO_EQ_BAND_156 $AUDIO_EQ_BAND_220 $AUDIO_EQ_BAND_331 $AUDIO_EQ_BAND_440 $AUDIO_EQ_BAND_622 $AUDIO_EQ_BAND_880 $AUDIO_EQ_BAND_1250 $AUDIO_EQ_BAND_1750 $AUDIO_EQ_BAND_2500 $AUDIO_EQ_BAND_3500 $AUDIO_EQ_BAND_5000 $AUDIO_EQ_BAND_10000 $AUDIO_EQ_BAND_20000
		plugin_conf_eq $1 $AUDIO_EQ_BAND_50 $AUDIO_EQ_BAND_100 $AUDIO_EQ_BAND_156 $AUDIO_EQ_BAND_220 $AUDIO_EQ_BAND_331 $AUDIO_EQ_BAND_440 $AUDIO_EQ_BAND_622 $AUDIO_EQ_BAND_880 $AUDIO_EQ_BAND_1250 $AUDIO_EQ_BAND_1750 $AUDIO_EQ_BAND_2500 $AUDIO_EQ_BAND_3500 $AUDIO_EQ_BAND_5000 $AUDIO_EQ_BAND_10000 $AUDIO_EQ_BAND_20000
	fi
}

plugin_reset_eq()
{
	if [ "$1" != "-q" ]; then 
		echo "-- Reset LADSPA equalizer plugin to 0 dB"
	fi

	plugin_conf_eq "-q" 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
}

#args: 50hz, 100hz, 156hz, 220hz, 311hz, 440hz, 622hz, 880hz, 1250hz, 1750hz, 25000hz, 3500hz, 5000hz, 10000hz, 20000hz
plugin_conf_eq()
{
	if [ "$1" == "-q" ]; then
		shift
	else
		echo "-- configure LADSPA equalizer plugin with controls [ $@ ]"
	fi

	i=1
	for db in "$@"
	do
		VALUE=`dbToValue $db -70`
		# FIXME: amixer: Control equalizer open error: Operation not permitted from service
		#amixer -q -D equalizer cset numid=$i $VALUE
		i=$((i+1))
	done
}

plugin_add_eq()
{
 	cat << EOF >> /etc/asound.conf

# LADSPA mbeq plugin
ctl.equalizer {
    type equal
    library "/usr/lib/ladspa/mbeq_1197.so"
    module "mbeq"
}
pcm.plugin_equalizer_f {
    type ladspa
    slave.pcm "$PCM_EQUALIZER_PLUGIN"
    path "/usr/lib/ladspa"
    plugins [{
        label mbeq
        id 1197
        input {
            # 50hz, 100hz, 156hz, 220hz, 311hz, 440hz, 622hz, 880hz, 1250hz, 1750hz, 25000hz, 3500hz, 5000hz, 10000hz, 20000hz
            controls [ $@ ]
        }
    }]
}
pcm.plugin_equalizer {
   type equal
   slave.pcm "$PCM_EQUALIZER_PLUGIN"
   library "/usr/lib/ladspa/mbeq_1197.so"
   module "mbeq"
}
pcm.equalizer {
   type plug
   slave.pcm plugin_equalizer
}
EOF
}


# args: peak limit, release time, fast ratio, ratio
# plugin_set_compressor()
# {
# 	if [ -z "$@" ]; then usage; fi

# 	CHECK=`grep "label dysonCompress" /etc/asound.conf`
# 	if [ -z "$CHECK" ]; then
# 		plugin_add_compressor $@
# 	else
# 		CTL="$@"
# 		echo "-- configure LADSPA compressor plugin with controls [ $CTL ]"
# 		sed -i.bak -E "s/controls.*# compressor ctl/controls [ ${CTL} ] # compressor ctl/g" /etc/asound.conf
# 	fi
# }
plugin_add_compressor()
{

	if [ "$1" == "-q" ]; then
		shift
	else
		echo "-- configure LADSPA compressor plugin with controls [ $@ ]"
	fi

	cat << EOF >> /etc/asound.conf 

# LADSPA compressor plugin
pcm.plugin_compressor {
    type ladspa
    slave.pcm "$PCM_COMPRESSOR_PLUGIN"
    path "/usr/lib/ladspa"
    plugins [{
        label dysonCompress
        id 1403
        input {
            # peak limit, release time, fast ratio, ratio
            controls [ $@ ]
        }
    }]
}
pcm.compressor {
   type plug
   slave.pcm plugin_compressor
}
EOF
}

# args: input gain, limit, release time
# plugin_set_limiter()
# {
# 	if [ -z "$@" ]; then usage; fi

# 	CHECK=`grep "label fastLookaheadLimiter" /etc/asound.conf`
# 	if [ -z "$CHECK" ]; then
# 		plugin_add_limiter $@
# 	else
# 		CTL="$@"
# 		echo "-- configure LADSPA limiter plugin with controls [ $CTL ]"
# 		sed -i.bak -E "s/controls.*# limiter ctl/controls [ ${CTL} ] # limiter ctl/g" /etc/asound.conf
# 	fi
# }
plugin_add_limiter()
{

	if [ "$1" == "-q" ]; then
		shift
	else
		echo "-- configure LADSPA limiter plugin with controls [ $@ ]"
	fi

 	cat << EOF >> /etc/asound.conf

# LADSPA limiter plugin
pcm.plugin_limiter {
    type ladspa
    slave.pcm "$PCM_LIMITER_PLUGIN"
    path "/usr/lib/ladspa"
    plugins [{
        label fastLookaheadLimiter
        id 1913
        input {
            # input gain, limit, release time, attenuation
            controls [ $@ ]
        }
    }]
}
pcm.limiter {
   type plug
   slave.pcm plugin_limiter
}
EOF
}




# ----------------------------------------------------------------------------------------------------------------------
#	ONBOARD (bcm2835 ALSA)
# ----------------------------------------------------------------------------------------------------------------------

doonboard()
{
	case $1 in
		setup )
			onboard_config
			;;
		reset )
			onboard_reset
			;;
		status )
			onboard_status
			;;
		gain|volume )
			onboard_volume $2
			;;
		output )
			onboard_route $2
			;;
		* )
			usage
			;;
	esac
}

onboard_reset()
{
	if [ "$1" != "-q" ]; then 
		echo "-- Reseting the onboard audio to -20 dB, automatic output"
	fi

	onboard_volume -20
	onboard_route auto
}

onboard_config()
{
	if [ "$1" != "-q" ]; then 
		echo "-- configure the onboard audio"
		echo "-- set onbaord volume to $ONBOARD_AUDIO_VOLUME dB"
		echo "-- set onboard route to $ONBOARD_AUDIO_OUTPUT"
	fi

	onboard_volume $ONBOARD_AUDIO_VOLUME
	onboard_route $ONBOARD_AUDIO_OUTPUT
}

onboard_volume()
{
	VALUE=`dbToPercent $1 -10239 400`
	amixer -q -Dhw:ALSA cset name='PCM Playback Volume' $VALUE%
}

onboard_route()
{
	case $1 in
		"hdmi" )
			ROUTE=2
			;;
		"jack" )
			ROUTE=1
			;;
		* )
			ROUTE=0
			;;
	esac

	amixer -q -Dhw:ALSA cset name='PCM Playback Route' $ROUTE
}

onboard_status()
{
	HAVE_ALSA=`docheck onboard`
	if [[ -z "$HAVE_ALSA" ]]; then
		echo "onboard outputs not found"
		exit 1
	fi

	VALUE=`amixer -Dhw:ALSA cget name='PCM Playback Volume' | pcregrep -o1 -e ": values=(-?\d+)"`
	ROUTE=`amixer -Dhw:ALSA cget name='PCM Playback Route' | pcregrep -o1 -e ": values=(\d)"`

	case $ROUTE in
		2 )
			ROUTE="digital HDMI"
			;;
		1 )
			ROUTE="analogue (headphone jack)"
			;;
		* )
			ROUTE="automatic"
			;;
	esac

	echo "onboard audio status"
	echo -e "\noutput:" && echo "-------"
	echo -e "PCM Playback Route ................. $ROUTE"
	echo -n "PCM Playback Volume ................ " && printDB `awk "BEGIN {print ($VALUE/100)}"`
}




# ----------------------------------------------------------------------------------------------------------------------
#	ReSpeaker 2-Mics
# ----------------------------------------------------------------------------------------------------------------------

dorespeaker()
{
	case $1 in
		setup )
			respeaker_config
			;;
		reset )
			respeaker_reset
			;;
		status )
			respeaker_status
			;;
		gain|volume )
			respeaker_volume $2 $3
			;;
		* )
			usage
			;;
	esac
}

respeaker_reset()
{
	if [ "$1" != "-q" ]; then 
		echo "-- Reseting the ReSpeaker 2-Mics audio card to -18 dB"
	fi

	respeaker_volume headphone -18
	respeaker_volume speaker -18
}

respeaker_config()
{
	if [ "$1" != "-q" ]; then 
		echo "-- configure the ReSpeaker 2-Mics audio card"
		echo "-- set Headphone volume to $RESPEAKER_HEADPHONE_VOL dB"
		echo "-- set Speaker volume to $RESPEAKER_SPEAKER_VOL dB"
	fi

	respeaker_volume headphone $RESPEAKER_HEADPHONE_VOL
	respeaker_volume speaker $RESPEAKER_SPEAKER_VOL
}

respeaker_volume()
{
	VALUE=`dbToValue $2 -121`
	amixer -q -c seeed2micvoicec cset name="${1^} Playback Volume" $VALUE
}

respeaker_status()
{
	HAVE_RESPEAKER=`docheck respeaker`
	if [[ -z "$HAVE_RESPEAKER" ]]; then
		echo "ReSpeaker 2-Mics audio card not found"
		exit 1
	fi

	HVALUE=`amixer -c seeed2micvoicec cget name='Headphone Playback Volume' | pcregrep -o1 -e ": values=(-?\d+)"`
	SVALUE=`amixer -c seeed2micvoicec cget name='Speaker Playback Volume' | pcregrep -o1 -e ": values=(-?\d+)"`


	echo "ReSpeaker 2-Mics audio card status"
	echo -e "\noutput:" && echo "-------"
	echo -n "Headphone Playback Volume .............. " && printDB `valueToDB $HVALUE -121`
	echo -n "Speaker Playback Volume ................ " && printDB `valueToDB $SVALUE -121`
}




# ----------------------------------------------------------------------------------------------------------------------
#	BLUE ICILE AUDIO CARD
# ----------------------------------------------------------------------------------------------------------------------

doicicle()
{
	case $1 in
		setup )
			icicle_config
			;;
		reset )
			icicle_reset
			;;
		status )
			icicle_status
			;;
		* )
			usage
			;;
	esac
}

icicle_reset()
{
	if [ "$1" != "-q" ]; then 
		echo "-- Reseting the Blue Icicle audio card to 0 dB"
	fi

	amixer -q -Dhw:Icicle cset name='Mic Capture Volume' 8
}

icicle_config()
{
	if [ -z "$BLUEICICLE_IN_GAIN" ]; then return; fi

	VALUE=`dbToValue $BLUEICICLE_IN_GAIN -8`

	if [ "$1" != "-q" ]; then 
		echo "-- configure the Blue Icicle audio card"
		echo "-- set ALSA mixer to $VALUE ($BLUEICICLE_IN_GAIN dB)"
	fi

	amixer -q -Dhw:Icicle cset name='Mic Capture Volume' $VALUE
}

icicle_status()
{
	HAVE_ICICLE=`docheck icicle`
	if [[ -z "$HAVE_ICICLE" ]]; then
		echo "Blue Icicle audio card: device not found"
		exit 1
	fi

	VALUE=`amixer -Dhw:Icicle cget name='Mic Capture Volume' | pcregrep -o1 -e ": values=(\d+)"`

	echo "Blue Icicle audio card"
	echo -e "\ninput:" && echo "------"
	echo -n "mic capture volume ................ " && printDB `valueToDB $VALUE -8`
}




# ----------------------------------------------------------------------------------------------------------------------
#	CIRRUS LOGIC / WOLFSON AUDIO CARD
# ----------------------------------------------------------------------------------------------------------------------

docirrus()
{
	case $1 in
		setup )
			cirrus_config
			;;
		reset )
			cirrus_reset
			;;
		check )
			cirrus_check $2
			;;
		status )
			cirrus_check all
			;;
		open )
			cirrus_open $2
			;;
		close )
			cirrus_close $2
			;;
		gain|volume )
			cirrus_volume $2 $3
			;;
		* )
			usage
			;;
	esac
}

cirrus_config()
{

	cirrus_reset $1

	if [ "$1" != "-q" ]; then 

		if [ "$CIRRUS_INPUT" ]; then

			INL_VOL=`dbToValue $CIRRUS_INL_VOL -32`
			INR_VOL=`dbToValue $CIRRUS_INR_VOL -32`
			INL_PGA=`dbToValue $CIRRUS_INL_PGA 0`
			INR_PGA=`dbToValue $CIRRUS_INR_PGA 0`
			INL_DIG=`dbToValue $CIRRUS_INL_DIG -64 0.5`
			INR_DIG=`dbToValue $CIRRUS_INR_DIG -64 0.5`

			echo "-- configure the Cirrus Logic/Wolfson audio card for '$CIRRUS_INPUT' input"
			echo "-- set ALSA mixer to $INL_VOL ($CIRRUS_INL_VOL dB)"
			echo "-- set ALSA mixer to $INR_VOL ($CIRRUS_INR_VOL dB)"
			echo "-- set ALSA mixer to $INL_PGA ($CIRRUS_INL_PGA dB)"
			echo "-- set ALSA mixer to $INR_PGA ($CIRRUS_INR_PGA dB)"
			echo "-- set ALSA mixer to $INL_DIG ($CIRRUS_INL_DIG dB)"
			echo "-- set ALSA mixer to $INR_DIG ($CIRRUS_INR_DIG dB)"
		fi

		OUTL_VOL=`dbToValue $CIRRUS_OUTL_VOL -32`
		OUTR_VOL=`dbToValue $CIRRUS_OUTR_VOL -32`

		echo "-- configure the Cirrus Logic/Wolfson audio card for headphone ($CIRRUS_HEADPHONE) output"
		echo "-- configure the Cirrus Logic/Wolfson audio card for lineout ($CIRRUS_LINE_OUT) output"
		echo "-- configure the Cirrus Logic/Wolfson audio card for speaker ($CIRRUS_SPEAKER) output"
		echo "-- configure the Cirrus Logic/Wolfson audio card for spdifout ($CIRRUS_SPDIF_OUT) output"
		echo "-- set interface output volume (L) to $OUTL_VOL ($CIRRUS_OUTL_VOL dB)"
		echo "-- set interface output volume (R) to $OUTR_VOL ($CIRRUS_OUTR_VOL dB)"
	fi

	# input
	if [ "$CIRRUS_INPUT" ]; then cirrus_open $CIRRUS_INPUT; fi
		
	# output
	if [ "$CIRRUS_HEADPHONE" == "yes" ]; then cirrus_open headphone; fi
	if [ "$CIRRUS_SPEAKER" == "yes" ]; then cirrus_open speaker; fi
	if [ "$CIRRUS_LINE_OUT" == "yes" ]; then cirrus_open lineout; fi
	if [ "$CIRRUS_SPDIF_OUT" == "yes" ]; then cirrus_open spdifout; fi
}

cirrus_volume()
{
	VALUE=`dbToValue $2 -32`

	if [ "$1" == "L" ]; then
		amixer -q -c RPiCirrus cset name='SPKOUTL Input 1 Volume' $VALUE
		amixer -q -c RPiCirrus cset name='HPOUT1L Input 1 Volume' $VALUE
		amixer -q -c RPiCirrus cset name='AIF2TX1 Input 1 Volume' $VALUE
	fi
	if [ "$1" == "R" ]; then
		amixer -q -c RPiCirrus cset name='SPKOUTR Input 1 Volume' $VALUE
		amixer -q -c RPiCirrus cset name='HPOUT1R Input 1 Volume' $VALUE
		amixer -q -c RPiCirrus cset name='AIF2TX2 Input 1 Volume' $VALUE
	fi
}

cirrus_reset_in()
{
	# in + filers
	amixer -q -c RPiCirrus cset name="IN1R Digital Volume" "128"
	amixer -q -c RPiCirrus cset name="IN1R Volume" "0"
	amixer -q -c RPiCirrus cset name="IN1 High Performance Switch" "off"
	amixer -q -c RPiCirrus cset name="IN2L Digital Volume" "128"
	amixer -q -c RPiCirrus cset name="IN2R Digital Volume" "128"
	amixer -q -c RPiCirrus cset name="IN2L Volume" "0"
	amixer -q -c RPiCirrus cset name="IN2R Volume" "0"
	amixer -q -c RPiCirrus cset name="IN2 High Performance Switch" "off"
	amixer -q -c RPiCirrus cset name="IN3L Digital Volume" "128"
	amixer -q -c RPiCirrus cset name="IN3R Digital Volume" "128"
	amixer -q -c RPiCirrus cset name="IN3L Volume" "0"
	amixer -q -c RPiCirrus cset name="IN3R Volume" "0"
	amixer -q -c RPiCirrus cset name="IN3 High Performance Switch" "off"
	amixer -q -c RPiCirrus cset name="Line Input Micbias" "off"
	amixer -q -c RPiCirrus cset name="LHPF1 Coefficients" "0,0"
	amixer -q -c RPiCirrus cset name="LHPF2 Coefficients" "0,0"
	amixer -q -c RPiCirrus cset name="LHPF3 Coefficients" "0,0"
	amixer -q -c RPiCirrus cset name="LHPF4 Coefficients" "0,0"
	amixer -q -c RPiCirrus cset name="LHPF1 Input 1" "None"
	amixer -q -c RPiCirrus cset name="LHPF2 Input 1" "None"
	amixer -q -c RPiCirrus cset name="LHPF1 Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF2 Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF1 Input 2" "None"
	amixer -q -c RPiCirrus cset name="LHPF2 Input 2" "None"
	amixer -q -c RPiCirrus cset name="LHPF1 Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF2 Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF1 Input 3" "None"
	amixer -q -c RPiCirrus cset name="LHPF2 Input 3" "None"
	amixer -q -c RPiCirrus cset name="LHPF1 Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF2 Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF1 Input 4" "None"
	amixer -q -c RPiCirrus cset name="LHPF2 Input 4" "None"
	amixer -q -c RPiCirrus cset name="LHPF1 Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF2 Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF1TX1 Input 1" "None"
	amixer -q -c RPiCirrus cset name="AIF1TX2 Input 1" "None"
	amixer -q -c RPiCirrus cset name="AIF1TX1 Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF1TX2 Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF1TX1 Input 2" "None"
	amixer -q -c RPiCirrus cset name="AIF1TX2 Input 2" "None"
	amixer -q -c RPiCirrus cset name="AIF1TX1 Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF1TX2 Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF1TX1 Input 3" "None"
	amixer -q -c RPiCirrus cset name="AIF1TX2 Input 3" "None"
	amixer -q -c RPiCirrus cset name="AIF1TX1 Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF1TX2 Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF1TX1 Input 4" "None"
	amixer -q -c RPiCirrus cset name="AIF1TX2 Input 4" "None"
	amixer -q -c RPiCirrus cset name="AIF1TX1 Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF1TX2 Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF2TX1 Input 1" "None"
	amixer -q -c RPiCirrus cset name="AIF2TX2 Input 1" "None"
	amixer -q -c RPiCirrus cset name="AIF2TX1 Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF2TX2 Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF2TX1 Input 2" "None"
	amixer -q -c RPiCirrus cset name="AIF2TX2 Input 2" "None"
	amixer -q -c RPiCirrus cset name="AIF2TX1 Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF2TX2 Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF2TX1 Input 3" "None"
	amixer -q -c RPiCirrus cset name="AIF2TX2 Input 3" "None"
	amixer -q -c RPiCirrus cset name="AIF2TX1 Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF2TX2 Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF2TX1 Input 4" "None"
	amixer -q -c RPiCirrus cset name="AIF2TX2 Input 4" "None"
	amixer -q -c RPiCirrus cset name="AIF2TX1 Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="AIF2TX2 Input 4 Volume" "32"
}

cirrus_reset_out()
{
	# out
	amixer -q -c RPiCirrus cset name="HPOUT2 Digital Switch" "off"
	amixer -q -c RPiCirrus cset name="HPOUT2 Digital Volume" "128"
	amixer -q -c RPiCirrus cset name="HPOUT2L Input 1" "None"
	amixer -q -c RPiCirrus cset name="HPOUT2R Input 1" "None"
	amixer -q -c RPiCirrus cset name="HPOUT2L Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT2R Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT2L Input 2" "None"
	amixer -q -c RPiCirrus cset name="HPOUT2R Input 2" "None"
	amixer -q -c RPiCirrus cset name="HPOUT2L Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT2R Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT2L Input 3" "None"
	amixer -q -c RPiCirrus cset name="HPOUT2R Input 3" "None"
	amixer -q -c RPiCirrus cset name="HPOUT2L Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT2R Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT2L Input 4" "None"
	amixer -q -c RPiCirrus cset name="HPOUT2R Input 4" "None"
	amixer -q -c RPiCirrus cset name="HPOUT2L Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT2R Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT1 Digital Switch" "off"
	amixer -q -c RPiCirrus cset name="HPOUT1 Digital Volume" "128"
	amixer -q -c RPiCirrus cset name="HPOUT1L Input 1" "None"
	amixer -q -c RPiCirrus cset name="HPOUT1R Input 1" "None"
	amixer -q -c RPiCirrus cset name="HPOUT1L Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT1R Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT1L Input 2" "None"
	amixer -q -c RPiCirrus cset name="HPOUT1R Input 2" "None"
	amixer -q -c RPiCirrus cset name="HPOUT1L Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT1R Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT1L Input 3" "None"
	amixer -q -c RPiCirrus cset name="HPOUT1R Input 3" "None"
	amixer -q -c RPiCirrus cset name="HPOUT1L Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT1R Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT1L Input 4" "None"
	amixer -q -c RPiCirrus cset name="HPOUT1R Input 4" "None"
	amixer -q -c RPiCirrus cset name="HPOUT1L Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="HPOUT1R Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="Speaker Digital Switch" "off"
	amixer -q -c RPiCirrus cset name="Speaker Digital Volume" "128"
	amixer -q -c RPiCirrus cset name="SPKOUTL Input 1" "None"
	amixer -q -c RPiCirrus cset name="SPKOUTR Input 1" "None"
	amixer -q -c RPiCirrus cset name="SPKOUTL Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="SPKOUTR Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="SPKOUTL Input 2" "None"
	amixer -q -c RPiCirrus cset name="SPKOUTR Input 2" "None"
	amixer -q -c RPiCirrus cset name="SPKOUTL Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="SPKOUTR Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="SPKOUTL Input 3" "None"
	amixer -q -c RPiCirrus cset name="SPKOUTR Input 3" "None"
	amixer -q -c RPiCirrus cset name="SPKOUTL Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="SPKOUTR Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="SPKOUTL Input 4" "None"
	amixer -q -c RPiCirrus cset name="SPKOUTR Input 4" "None"
	amixer -q -c RPiCirrus cset name="SPKOUTL Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="SPKOUTR Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF3 Input 1" "None"
	amixer -q -c RPiCirrus cset name="LHPF4 Input 1" "None"
	amixer -q -c RPiCirrus cset name="LHPF3 Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF4 Input 1 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF3 Input 2" "None"
	amixer -q -c RPiCirrus cset name="LHPF4 Input 2" "None"
	amixer -q -c RPiCirrus cset name="LHPF3 Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF4 Input 2 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF3 Input 3" "None"
	amixer -q -c RPiCirrus cset name="LHPF4 Input 3" "None"
	amixer -q -c RPiCirrus cset name="LHPF3 Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF4 Input 3 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF3 Input 4" "None"
	amixer -q -c RPiCirrus cset name="LHPF4 Input 4" "None"
	amixer -q -c RPiCirrus cset name="LHPF3 Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="LHPF4 Input 4 Volume" "32"
	amixer -q -c RPiCirrus cset name="Min Sample Rate" "off"
	amixer -q -c RPiCirrus cset name="Max Sample Rate" "off"
	amixer -q -c RPiCirrus cset name="Tx Source" "AIF"
}

cirrus_reset()
{
	if [ "$1" != "-q" ]; then 
		echo "-- Reseting the Cirrus Logic/Wolfson audio card to default path"
	fi

	cirrus_reset_in
	cirrus_reset_out
}

cirrus_check()
{
	HAVE_CIRRUS=`docheck cirrus`
	if [[ -z "$HAVE_CIRRUS" ]]; then
		echo "Cirrus Logic/Wolfson audio card: device not found"
		exit 1
	fi

	case $@ in
		mic       ) SWITCH="IN1 High Performance Switch" ;;
		dmic      ) SWITCH="IN2 High Performance Switch" ;;
		micbias   ) SWITCH="Line Input Micbias" ;;
		linein    ) SWITCH="IN3 High Performance Switch" ;;
		spdifin   ) SWITCH="AIF1TX1 Input 1" ;;
		speaker   ) SWITCH="Speaker Digital Switch" ;;
		headphone ) SWITCH="HPOUT1 Digital Switch" ;;
		lineout   ) SWITCH="HPOUT2 Digital Switch" ;;
		spdifout  ) SWITCH="AIF2TX1 Input 1" ;;
		all )

			case $CIRRUS_INPUT in
				mic )
					PGA=1
					DIG=1
					;;
				dmic )
					PGA=2
					DIG=2
					;;
				* )
					PGA=3
					DIG=3
					;;
			esac

			INL_VOL=`valueToDB $(cirrus_in_get "AIF1TX1 Input 1 Volume") -32`
			INR_VOL=`valueToDB $(cirrus_in_get "AIF1TX2 Input 1 Volume") -32`
			INL_PGA=`valueToDB $(cirrus_in_get "IN${PGA}L Volume") 0`
			INR_PGA=`valueToDB $(cirrus_in_get "IN${PGA}R Volume") 0`
			INL_DIG=`valueToDB $(cirrus_in_get "IN${DIG}L Digital Volume") -64 0.5`
			INR_DIG=`valueToDB $(cirrus_in_get "IN${DIG}R Digital Volume") -64 0.5`
			OUTL_VOL=`valueToDB $(cirrus_in_get "HPOUT1L Input 1 Volume") -32`
			OUTR_VOL=`valueToDB $(cirrus_in_get "HPOUT1R Input 1 Volume") -32`


			echo "Cirrus Logic/Wolfson audio card"
			echo -e "\ninput:" && echo "------"
			echo -n "Digital MEMS microphones (DMIC) ... " && cirrus_check dmic
			echo -n "Mic (HeadSet) ..................... " && cirrus_check mic
			echo -n "Line In  .......................... " && cirrus_check linein
			echo -n "Line In (bias voltage) ............ " && cirrus_check micbias
			echo -n "SPDIF In .......................... " && cirrus_check spdifin
			echo -e ""
			echo -n "interface input volume (L) ........ " && printDB $INL_VOL
			echo -n "interface input volume (R) ........ " && printDB $INR_VOL
			echo -n "input PGA gain (L) ................ " && printDB $INL_PGA
			echo -n "input PGA gain (R) ................ " && printDB $INR_PGA
			echo -n "digital input gain (L) ............ " && printDB $INL_DIG
			echo -n "digital input gain (R) ............ " && printDB $INR_DIG
			echo -e "\noutput:" && echo "-------"
			echo -n "Headphone  ........................ " && cirrus_check headphone
			echo -n "Line Out .......................... " && cirrus_check lineout
			echo -n "Speakers  ......................... " && cirrus_check speaker
			echo -n "SPDIF Out ......................... " && cirrus_check spdifout
			echo -e ""
			echo -n "interface output volume (L) ....... " && printDB $OUTL_VOL
			echo -n "interface output volume (R) ....... " && printDB $OUTR_VOL
			exit 0
			;;
		* )
			usage
			;;
	esac

	# spdifin : #24 'AIF2RX1', spdifout : #16 'AIF1RX1'
	ACTIVE=`amixer -c RPiCirrus cget name="$SWITCH" | grep -E ": values=on|: values=1|: values=16|: values=24"`

	if [[ "$1" == "linein" && `cirrus_check micbias` == "ON" ]]; then ACTIVE=""; fi
	if [ "$ACTIVE" ]; then echo "ON"; else echo "OFF"; fi
}

cirrus_open()
{

	if [ -z "$CIRRUS_INL_VOL" ]; then CIRRUS_INL_VOL="0"; fi
	if [ -z "$CIRRUS_INR_VOL" ]; then CIRRUS_INR_VOL="0"; fi
	if [ -z "$CIRRUS_INL_PGA" ]; then CIRRUS_INL_PGA="0"; fi
	if [ -z "$CIRRUS_INR_PGA" ]; then CIRRUS_INR_PGA="0"; fi
	if [ -z "$CIRRUS_INL_DIG" ]; then CIRRUS_INL_DIG="-6"; fi
	if [ -z "$CIRRUS_INR_DIG" ]; then CIRRUS_INR_DIG="-6"; fi
	if [ -z "$CIRRUS_OUTL_VOL" ]; then CIRRUS_OUTL_VOL="0"; fi
	if [ -z "$CIRRUS_OUTR_VOL" ]; then CIRRUS_OUTR_VOL="0"; fi


	case $@ in
		# Record_from_Headset.sh
		mic ) 
			cirrus_reset_in
			amixer -q -c RPiCirrus cset name='IN1R Volume' `dbToValue $CIRRUS_INR_PGA 0`
			amixer -q -c RPiCirrus cset name='IN1R Digital Volume' `dbToValue $CIRRUS_INR_DIG -64 0.5`
			amixer -q -c RPiCirrus cset name="LHPF1 Mode" High-pass
			amixer -q -c RPiCirrus cset name="LHPF2 Mode" High-pass
			amixer -q -c RPiCirrus cset name="LHPF1 Coefficients" 240,3
			amixer -q -c RPiCirrus cset name="LHPF2 Coefficients" 240,3
			amixer -q -c RPiCirrus cset name='LHPF1 Input 1' IN1R
			amixer -q -c RPiCirrus cset name="LHPF2 Input 1" IN1R
			amixer -q -c RPiCirrus cset name="LHPF1 Input 1 Volume" 32
			amixer -q -c RPiCirrus cset name="LHPF2 Input 1 Volume" 32
			amixer -q -c RPiCirrus cset name='AIF1TX1 Input 1' LHPF1
			amixer -q -c RPiCirrus cset name='AIF1TX2 Input 1' LHPF2
			amixer -q -c RPiCirrus cset name='AIF1TX1 Input 1 Volume' `dbToValue $CIRRUS_INL_VOL -32`
			amixer -q -c RPiCirrus cset name='AIF1TX2 Input 1 Volume' `dbToValue $CIRRUS_INR_VOL -32`
			amixer -q -c RPiCirrus cset name='IN1 High Performance Switch' on
			;;
		# Record_from_DMIC.sh
		dmic )
			cirrus_reset_in
			amixer -q -c RPiCirrus cset name="IN2L Volume" 0
			amixer -q -c RPiCirrus cset name="IN2R Volume" 0
			amixer -q -c RPiCirrus cset name='IN2L Digital Volume' `dbToValue $CIRRUS_INL_DIG -64 0.5`
			amixer -q -c RPiCirrus cset name='IN2R Digital Volume' `dbToValue $CIRRUS_INR_DIG -64 0.5`
			amixer -q -c RPiCirrus cset name='LHPF1 Input 1' IN2L
			amixer -q -c RPiCirrus cset name='LHPF2 Input 1' IN2R
			amixer -q -c RPiCirrus cset name='LHPF1 Mode' High-pass
			amixer -q -c RPiCirrus cset name='LHPF2 Mode' High-pass
			amixer -q -c RPiCirrus cset name='LHPF1 Coefficients' 240,3
			amixer -q -c RPiCirrus cset name='LHPF2 Coefficients' 240,3
			amixer -q -c RPiCirrus cset name="LHPF1 Input 1 Volume" 32
			amixer -q -c RPiCirrus cset name="LHPF2 Input 1 Volume" 32
			amixer -q -c RPiCirrus cset name='AIF1TX1 Input 1' LHPF1
			amixer -q -c RPiCirrus cset name='AIF1TX2 Input 1' LHPF2
			amixer -q -c RPiCirrus cset name='AIF1TX1 Input 1 Volume' `dbToValue $CIRRUS_INL_VOL -32`
			amixer -q -c RPiCirrus cset name='AIF1TX2 Input 1 Volume' `dbToValue $CIRRUS_INR_VOL -32`
			amixer -q -c RPiCirrus cset name='IN2 High Performance Switch' on
			;;
		# Record_from_lineIn_Micbias.sh
		micbias )
			cirrus_reset_in
			amixer -q -c RPiCirrus cset name="Line Input Micbias" on
			amixer -q -c RPiCirrus cset name="IN3 High Performance Switch" on
			amixer -q -c RPiCirrus cset name="IN3L Volume" `dbToValue $CIRRUS_INL_PGA 0`
			amixer -q -c RPiCirrus cset name="IN3R Volume" `dbToValue $CIRRUS_INR_PGA 0`
			amixer -q -c RPiCirrus cset name="IN3L Digital Volume" `dbToValue $CIRRUS_INL_DIG -64 0.5`
			amixer -q -c RPiCirrus cset name="IN3R Digital Volume" `dbToValue $CIRRUS_INR_DIG -64 0.5`
			amixer -q -c RPiCirrus cset name="LHPF1 Mode" High-pass
			amixer -q -c RPiCirrus cset name="LHPF2 Mode" High-pass
			amixer -q -c RPiCirrus cset name="LHPF1 Coefficients" 240,3
			amixer -q -c RPiCirrus cset name="LHPF2 Coefficients" 240,3
			amixer -q -c RPiCirrus cset name="LHPF1 Input 1" IN3L
			amixer -q -c RPiCirrus cset name="LHPF2 Input 1" IN3R
			amixer -q -c RPiCirrus cset name="LHPF1 Input 1 Volume" 32
			amixer -q -c RPiCirrus cset name="LHPF2 Input 1 Volume" 32
			amixer -q -c RPiCirrus cset name="AIF1TX1 Input 1" LHPF1
			amixer -q -c RPiCirrus cset name="AIF1TX2 Input 1" LHPF2
			amixer -q -c RPiCirrus cset name='AIF1TX1 Input 1 Volume' `dbToValue $CIRRUS_INL_VOL -32`
			amixer -q -c RPiCirrus cset name='AIF1TX2 Input 1 Volume' `dbToValue $CIRRUS_INR_VOL -32`
			;;
		# Record_from_lineIn.sh
		linein )
			cirrus_reset_in
			amixer -q -c RPiCirrus cset name="IN3 High Performance Switch" on
			amixer -q -c RPiCirrus cset name='IN3L Volume' `dbToValue $CIRRUS_INL_PGA 0`
			amixer -q -c RPiCirrus cset name='IN3R Volume' `dbToValue $CIRRUS_INR_PGA 0`
			amixer -q -c RPiCirrus cset name='IN3L Digital Volume' `dbToValue $CIRRUS_INL_DIG -64 0.5`
			amixer -q -c RPiCirrus cset name='IN3R Digital Volume' `dbToValue $CIRRUS_INR_DIG -64 0.5`
			amixer -q -c RPiCirrus cset name="LHPF1 Mode" High-pass
			amixer -q -c RPiCirrus cset name="LHPF2 Mode" High-pass
			amixer -q -c RPiCirrus cset name="LHPF1 Coefficients" 240,3
			amixer -q -c RPiCirrus cset name="LHPF2 Coefficients" 240,3
			amixer -q -c RPiCirrus cset name="LHPF1 Input 1" IN3L
			amixer -q -c RPiCirrus cset name="LHPF2 Input 1" IN3R
			amixer -q -c RPiCirrus cset name="LHPF1 Input 1 Volume" 32
			amixer -q -c RPiCirrus cset name="LHPF2 Input 1 Volume" 32
			amixer -q -c RPiCirrus cset name="AIF1TX1 Input 1" LHPF1
			amixer -q -c RPiCirrus cset name="AIF1TX2 Input 1" LHPF2
			amixer -q -c RPiCirrus cset name='AIF1TX1 Input 1 Volume' `dbToValue $CIRRUS_INL_VOL -32`
			amixer -q -c RPiCirrus cset name='AIF1TX2 Input 1 Volume' `dbToValue $CIRRUS_INR_VOL -32`
			;;
		# Record_from_SPDIF.sh
		spdifin )
			amixer -q -c RPiCirrus cset name="Min Sample Rate" 32kHz
			amixer -q -c RPiCirrus cset name="AIF1TX1 Input 1" AIF2RX1
			amixer -q -c RPiCirrus cset name='AIF1TX1 Input 1 Volume' `dbToValue $CIRRUS_INL_VOL -32`
			amixer -q -c RPiCirrus cset name="AIF1TX2 Input 1" AIF2RX2
			amixer -q -c RPiCirrus cset name='AIF1TX2 Input 1 Volume' `dbToValue $CIRRUS_INR_VOL -32`
			;;
		# Playback_to_Speakers.sh
		speaker )
			amixer -q -c RPiCirrus cset name='Speaker Digital Volume' 128
			amixer -q -c RPiCirrus cset name='SPKOUTL Input 1' None
			amixer -q -c RPiCirrus cset name='SPKOUTR Input 1' None
			amixer -q -c RPiCirrus cset name='SPKOUTL Input 2' None
			amixer -q -c RPiCirrus cset name='SPKOUTR Input 2' None
			amixer -q -c RPiCirrus cset name='SPKOUTL Input 1' AIF1RX1
			amixer -q -c RPiCirrus cset name='SPKOUTR Input 1' AIF1RX2
			amixer -q -c RPiCirrus cset name='SPKOUTL Input 1 Volume' `dbToValue $CIRRUS_OUTL_VOL -32`
			amixer -q -c RPiCirrus cset name='SPKOUTR Input 1 Volume' `dbToValue $CIRRUS_OUTR_VOL -32`
			amixer -q -c RPiCirrus cset name='Speaker Digital Switch' on
			;;
		# Playback_to_Headset.sh
		headphone )
			amixer -q -c RPiCirrus cset name='HPOUT1 Digital Volume' 116
			amixer -q -c RPiCirrus cset name='HPOUT1L Input 1' None
			amixer -q -c RPiCirrus cset name='HPOUT1R Input 1' None
			amixer -q -c RPiCirrus cset name='HPOUT1L Input 2' None
			amixer -q -c RPiCirrus cset name='HPOUT1R Input 2' None
			amixer -q -c RPiCirrus cset name='HPOUT1L Input 1' AIF1RX1
			amixer -q -c RPiCirrus cset name='HPOUT1R Input 1' AIF1RX2
			amixer -q -c RPiCirrus cset name='HPOUT1L Input 1 Volume' `dbToValue $CIRRUS_OUTL_VOL -32`
			amixer -q -c RPiCirrus cset name='HPOUT1R Input 1 Volume' `dbToValue $CIRRUS_OUTR_VOL -32`
			amixer -q -c RPiCirrus cset name='HPOUT1 Digital Switch' on
			;;
		# Playback_to_Lineout.sh
		lineout )
			amixer -q -c RPiCirrus cset name='HPOUT2L Input 1' None
			amixer -q -c RPiCirrus cset name='HPOUT2R Input 1' None
			amixer -q -c RPiCirrus cset name='HPOUT2L Input 2' None
			amixer -q -c RPiCirrus cset name='HPOUT2R Input 2' None
			amixer -q -c RPiCirrus cset name='HPOUT2L Input 1' AIF1RX1
			amixer -q -c RPiCirrus cset name='HPOUT2R Input 1' AIF1RX2
			amixer -q -c RPiCirrus cset name='HPOUT1L Input 1 Volume' `dbToValue $CIRRUS_OUTL_VOL -32`
			amixer -q -c RPiCirrus cset name='HPOUT1R Input 1 Volume' `dbToValue $CIRRUS_OUTR_VOL -32`
			amixer -q -c RPiCirrus cset name='HPOUT2 Digital Switch' on
			;;
		# Playback_to_SPDIF.sh
		spdifout )
			amixer -q -c RPiCirrus cset name="Min Sample Rate" 32kHz
			amixer -q -c RPiCirrus cset name='Tx Source' AIF
			amixer -q -c RPiCirrus cset name='AIF2TX1 Input 1' AIF1RX1
			amixer -q -c RPiCirrus cset name='AIF2TX2 Input 1' AIF1RX2
			amixer -q -c RPiCirrus cset name='AIF2TX1 Input 1 Volume' `dbToValue $CIRRUS_OUTL_VOL -32`
			amixer -q -c RPiCirrus cset name='AIF2TX2 Input 1 Volume' `dbToValue $CIRRUS_OUTR_VOL -32`
			;;
		* )
			usage
			;;
	esac
}

cirrus_close()
{
	case $@ in
		mic       )
			SWITCH="IN1 High Performance Switch"
			OUTPUT="AIF"
			;;
		dmic      )
			SWITCH="IN2 High Performance Switch"
			OUTPUT="AIF"
			;;
		micbias   )
			SWITCH="Line Input Micbias"
			OUTPUT="AIF"
			;;
		linein    )
			SWITCH="IN3 High Performance Switch"
			OUTPUT="AIF"
			;;
		spdifin   )
			SWITCH="SPDIF In Switch"
			OUTPUT="AIF"
			;;
		speaker   )
			SWITCH="Speaker Digital Switch"
			OUTPUT="SPK"
			;;
		headphone )
			SWITCH="HPOUT1 Digital Switch"
			OUTPUT="HPOUT1"
			;;
		lineout   )
			SWITCH="HPOUT2 Digital Switch"
			OUTPUT="HPOUT2"
			;;
		spdifout  )
			SWITCH="SPDIF Out Switch"
			OUTPUT="AIF"
			;;
		* )
			usage
			;;
	esac

	if [ "$SWITCH" ]; then
		amixer -q -c RPiCirrus cset name="$SWITCH" off
	fi
	if [ "$OUTPUT" == "AIF" ]; then
		amixer -q -c RPiCirrus cset name='AIF2TX1 Input 1' None
		amixer -q -c RPiCirrus cset name='AIF2TX2 Input 1' None
		amixer -q -c RPiCirrus cset name='AIF1TX1 Input 1' None
		amixer -q -c RPiCirrus cset name='AIF1TX2 Input 1' None
	fi
	if [ "$OUTPUT" == "HPOUT1" ]; then
		amixer -q -c RPiCirrus cset name='HPOUT1L Input 1' None
		amixer -q -c RPiCirrus cset name='HPOUT1R Input 1' None
		amixer -q -c RPiCirrus cset name='HPOUT1L Input 2' None
		amixer -q -c RPiCirrus cset name='HPOUT1R Input 2' None
	fi
	if [ "$OUTPUT" == "HPOUT2" ]; then
		amixer -q -c RPiCirrus cset name='HPOUT2L Input 1' None
		amixer -q -c RPiCirrus cset name='HPOUT2R Input 1' None
		amixer -q -c RPiCirrus cset name='HPOUT2L Input 2' None
		amixer -q -c RPiCirrus cset name='HPOUT2R Input 2' None
	fi
	if [ "$OUTPUT" == "SPK" ]; then
		amixer -q -c RPiCirrus cset name='SPKOUTL Input 1' None
		amixer -q -c RPiCirrus cset name='SPKOUTR Input 1' None
		amixer -q -c RPiCirrus cset name='SPKOUTL Input 2' None
		amixer -q -c RPiCirrus cset name='SPKOUTR Input 2' None
	fi
}

cirrus_in_get()
{
	echo `amixer -c RPiCirrus cget name="$1" | pcregrep -o1 -e ": values=(\d+)"`
}




# ----------------------------------------------------------------------------------------------------------------------
#	MAIN FUNCTIONS
# ----------------------------------------------------------------------------------------------------------------------

usage()
{
	echo "$0 {COMMAND} ..."
	echo ""
	echo "Audio card and plugin manager"
	echo "Supported audio card: onboard interfaces, Blue Icicle, Cirrus Logic/Wolfson, ReSpeaker 2-mic"
	echo 
	echo "  -h --help                              show this help"
	echo
	echo "  --test start|stop  pink|sine                     pink=use pink noise, sine=use sine wave"
	echo "  --monitor start|stop                             monitor the selected audio card input"
	echo "  --check onboard|cirrus|icicle                    test if the given audio card is connected"
	echo "  --setup                                quiet     configure the user (or default) requirements for all connected audio"
	echo "                                                   card and update ALSA configuration"
	echo 
	echo "  --master                               percent   set the sound card's master volume"
	echo "  --card                                 number    set the default sound card to the specified number"
	echo
	echo "  --plugin  setup|reset                            configure with user parameters or reset all ALSA plugins"
	echo "  --plugin  compressor                   * * * *   configure the LADSPA compressor plugin with the given controls"
	echo "  --plugin  limiter                      * * *     configure the LADSPA limiter plugin with the given controls"
	echo "  --plugin  equalizer                    * * ...   configure the LADSPA equalizer plugin with the given controls" 
	echo
	echo "  --onboard setup|reset                            configure or reset the onboard audio outputs"
	echo "  --onboard status                                 dump output status"
	echo "  --onboard volume                       dB        set the current onboard volume"
	echo "  --onboard output                       name      switch to the specified ouput (auto, jack, hdmi)"
	echo
	echo "  --respeaker setup|reset                          configure or reset the ReSpeaker 2-mic audio card"
	echo "  --respeaker status                               dump output status"
	echo "  --respeaker volume  headphone|speaker  dB        set the ReSpeaker 2-mic output volume"
	echo
	echo "  --icicle setup|reset                             configure or reset the Blue Icicle audio card"
	echo "  --icicle status                                  dump input status"
	echo
	echo "  --cirrus setup|reset                             configure or reset the Cirrus Logic/Wolfson audio card"
	echo "  --cirrus status                                  dump input or output status"
	echo "  --cirrus volume  L|R                   dB        set the Cirrus Logic/Wolfson output volume"
	echo "  --cirrus check|open|close              in|out    check or manage the given Cirrus Logic/Wolfson input or output"
	echo "                                                   where in is: dmic, mic, micbias, linein or spdifin"
	echo "                                                   where out is: speaker, headphone, lineout or spdifout"
	echo
	echo "  ------------------------------------------------------------------------------------------------------------"
	echo 
	echo "  plugin compressor controls"
	echo "    * peak limit in dB, range min=-30 max=0"
	echo "    * release time in seconds, range min=0 max=1"
	echo "    * fast ratio, range min=0 max=1"
	echo "    * ratio, range min=0 max=1 "
	echo
	echo "  plugin limiter controls"
	echo "    * input gain in dB, range min=-20 max=20"
	echo "    * limit in dB, range min=-20 max=0"
	echo "    * release time in second, range min=0.01 max=2.0"
	echo
	echo "  plugin EQ. controls, bands in dB, range min=-70 max=+30, default=0"
	echo "    * 50hz, 100hz, 156hz, 220hz, 311hz, 440hz, 622hz, 880hz, 1250hz, 1750hz, 25000hz, 3500hz, 5000hz, 10000hz, 20000hz"
	echo

	exit 1
}

case $1 in
	--setup )
		configure $2
		;;
	--master )
		setmaster $2
		;;
	--card )
		asound_conf $2
		;;
	--plugin )
		shift
		doplugin $@
		;;
	--check )
		docheck $2
		;;
	--test )
		dotest $2 $3
		;;
	--monitor )
		domonitoring $2
		;;
	--onboard )
		doonboard $2 $3
		;;
	--respeaker )
		dorespeaker $2 $3 $4
		;;
	--icicle )
		doicicle $2
		;;
	--cirrus )
		docirrus $2 $3 $4
		;;
	-h|* )
		usage
		;;
esac
exit $?
